from __future__ import print_function

import __builtin__
import functools
import inspect
import sys
import threading
import time
import traceback as tb

import Globals

def stacktrace(func):
	@functools.wraps(func)
	def wrapped(*args, **kwargs):
		print_bt(fund.__name__)
		return func(*args, **kwargs)

	return wrapped


def print_bt(name=None):
	if name is not None:
		dprint('=== {} backtrace:'.format(name))

	for line in tb.format_stack()[:-1]:
		dprint(line.strip())

	if name is not None:
		dprint('=== end {} backtrace'.format(name))


def dprint(*args, **kwargs):
	prefix = kwargs.pop('prefix', False)

	if prefix:
		prefix = 'SERVER' if Globals.server else 'CLIENT'
		print('[{}] ', end='')

	print(*args, file=sys.stderr)
	sys.stderr.flush()


def replace_builtin(name):
	def wrapper(f):
		original = getattr(__builtin__, name)

		@functools.wraps(f)
		def wrapped(*args, **kwargs):
			return f(original, *args, **kwargs)

		setattr(__builtin__, name, wrapped)
		return wrapped
	return wrapper


def print_iter(iterable):
	for item in iterable:
		dprint(str(item))
		yield item


def debug_iter(iterable):
	for item in iterable:
		yield item
		time.sleep(1)

class DebugLock(threading._RLock):

	def acquire(self):
		t = threading.current_thread()
		#  dprint('{} lock acquire'.format(t.name), prefix=True)
		print_bt('{} lock acquire'.format(t.name))
		return super(DebugLock, self).acquire()

	def release(self):
		t = threading.current_thread()
		dprint('{} released lock'.format(t.name))
		return super(DebugLock, self).release()

#  threading._RLock = DebugLock
